/* $Header: d:/cvsroot/tads/tads3/vmbiftad.h,v 1.2 1999/05/17 02:52:29 MJRoberts Exp $ */

/* Copyright (c) 1998 by Michael J. Roberts.  All Rights Reserved. */
/*
Name
  vmbiftad.h - function set definition - TADS function set
Function
  
Notes
  
Modified
  12/06/98 MJRoberts  - Creation
*/

#ifndef VMBIFTAD_H
#define VMBIFTAD_H

#include "os.h"
#include "vmbif.h"


/* ------------------------------------------------------------------------ */
/*
 *   Random number generator configuration.  Define one of the following
 *   configuration variables to select a random number generation
 *   algorithm:
 *   
 *   VMBIFTADS_RNG_LCG - linear congruential generator
 *.  VMBIFTADS_RNG_ISAAC - ISAAC (cryptographic hash generator) 
 */

/* use ISAAC */
#define VMBIFTADS_RNG_ISAAC


/* ------------------------------------------------------------------------ */
/*
 *   TADS function set built-in functions 
 */
class CVmBifTADS: public CVmBif
{
public:
    /*
     *   General functions 
     */
    static void datatype(VMG_ uint argc);
    static void getarg(VMG_ uint argc);
    static void firstobj(VMG_ uint argc);
    static void nextobj(VMG_ uint argc);
    static void randomize(VMG_ uint argc);
    static void rand(VMG_ uint argc);
    static void cvtstr(VMG_ uint argc);
    static void cvtnum(VMG_ uint argc);
    static void gettime(VMG_ uint argc);
    static void re_match(VMG_ uint argc);
    static void re_search(VMG_ uint argc);
    static void re_group(VMG_ uint argc);
    static void re_replace(VMG_ uint argc);
    static void savepoint(VMG_ uint argc);
    static void undo(VMG_ uint argc);
    static void save(VMG_ uint argc);
    static void restore(VMG_ uint argc);
    static void restart(VMG_ uint argc);
    static void get_max(VMG_ uint argc);
    static void get_min(VMG_ uint argc);
    static void make_string(VMG_ uint argc);

    /*
     *   Input/output functions 
     */
    static void say(VMG_ uint argc);
    static void file_open(VMG_ uint argc);
    static void file_close(VMG_ uint argc);
    static void file_tell(VMG_ uint argc);
    static void file_seek(VMG_ uint argc);
    static void file_seekeof(VMG_ uint argc);
    static void file_write(VMG_ uint argc);
    static void file_read(VMG_ uint argc);
    static void logging(VMG_ uint argc);
    static void clearscreen(VMG_ uint argc);
    static void more(VMG_ uint argc);
    static void input(VMG_ uint argc);
    static void inputkey(VMG_ uint argc);
    static void inputevent(VMG_ uint argc);
    static void inputdialog(VMG_ uint argc);
    static void askfile(VMG_ uint argc);
    static void timedelay(VMG_ uint argc);
    static void sysinfo(VMG_ uint argc);
    static void status_mode(VMG_ uint argc);
    static void status_right(VMG_ uint argc);
    static void res_exists(VMG_ uint argc);
    static void set_script_file(VMG_ uint argc);

protected:
    /* enumerate objects (common handler for firstobj and nextobj) */
    static void enum_objects(VMG_ uint argc, vm_obj_id_t start_obj);

    /* 
     *   Map a keystroke from raw (os_getc_raw) notation to a portable
     *   UTF-8 representation.
     *   
     *   - For backspace (ctrl-H or ASCII 127), we'll return '[bksp]'.
     *   
     *   - For ASCII 10 or 13, we'll return ASCII 10 ('\n')
     *   
     *   - For ASCII 9, we'll return ASCII 9 ('\t').
     *   
     *   - For other control characters, we'll return a name like
     *   '[ctrl-z]'.
     *   
     *   - For Escape, we'll return '[esc]'.
     *   
     *   - For ALT keys, we'll return a name like '[alt-x]'.
     *   
     *   - For CMD_xxx codes, we'll return a key name enclosed in square
     *   brackets; for example, for a left cursor arrow key, we'll return
     *   '[left]'.
     *   
     *   - For any non-control character that isn't a CMD_xxx command
     *   code, we'll map the character to UTF-8 and return the resulting
     *   single character (which might, of course, be more than one byte
     *   long).
     *   
     *   - For unknown CMD_xxx keys, we'll return '[?]'.
     *   
     *   Returns true if we found a valid mapping for the key, false if
     *   not (in which case the buffer will be filled in with '[?]'.  The
     *   return buffer is always null-terminated.  
     */
    static int map_raw_key(VMG_ char *buf, char c, char extc);
};


/* ------------------------------------------------------------------------ */
/*
 *   File context structure.  Each open file is represented by one of
 *   these structures; the function set globals keep an array of these for
 *   allocation on open requests.  
 */
struct CVmBifTadsFile
{
    /* close the file and release its resources */
    void close();
    
    /* the underlying system file handle */
    osfildef *fp;

    /* character mappers for reading and writing text */
    class CCharmapToUni *charmap_read;
    class CCharmapToLocal *charmap_write;

    /* flags */
    uint flags;
};

/* maximum number of open files */
#define VMBIFTADS_FILES_MAX  32

/*
 *   file flags 
 */
/* file is binary (the alternative is text) */
#define VMBIFTADS_FILE_BINARY  0x01


/* ------------------------------------------------------------------------ */
/*
 *   Global information for the TADS intrinsics.  We allocate this
 *   structure with the VM global variables - G_bif_tads_globals contains
 *   the structure.  
 */
class CVmBifTADSGlobals
{
public:
    CVmBifTADSGlobals();
    ~CVmBifTADSGlobals();

    /* regular expression parser */
    class CRegexParser *regex;

    /* open file context structures */
    CVmBifTadsFile fp[VMBIFTADS_FILES_MAX];

    /* -------------------------------------------------------------------- */
    /*
     *   Linear Congruential Random Number Generator state 
     */
#ifdef VMBIFTADS_RNG_LCG

    /* linear congruential generator seed value */
    long rand_seed;

#endif /* VMBIFTADS_RNG_LCG */

    /* -------------------------------------------------------------------- */
    /*
     *   ISAAC Random Number Generator state
     */
#ifdef VMBIFTADS_RNG_ISAAC

    /* ISAAC state structure */
    struct isaacctx *isaac_ctx;
    
#endif /* VMBIFTADS_RNG_ISAAC */
};

/* ------------------------------------------------------------------------ */
/*
 *   ISAAC Random Number Generator definitions
 */
#ifdef VMBIFTADS_RNG_ISAAC

#define ISAAC_RANDSIZL   (8)
#define ISAAC_RANDSIZ    (1<<ISAAC_RANDSIZL)

struct isaacctx
{
    ulong cnt;
    ulong rsl[ISAAC_RANDSIZ];
    ulong mem[ISAAC_RANDSIZ];
    ulong a;
    ulong b;
    ulong c;
};
#endif /* VMBIFTADS_RNG_ISAAC */

/* end of section protected against multiple inclusion */
#endif /* VMBIFTAD_H */


/* ------------------------------------------------------------------------ */
/*
 *   TADS function set vectors.  Define this only if VMBIF_DEFINE_VECTOR
 *   has been defined, so that this file can be included for the
 *   prototypes alone without defining the function vector.
 *   
 *   Note that this vector is specifically defined outside of the section
 *   of the file protected against multiple inclusion.  
 */
#ifdef VMBIF_DEFINE_VECTOR

/* TADS general data manipulation functions */
void (*G_bif_tadsgen[])(VMG_ uint) =
{
    &CVmBifTADS::datatype,
    &CVmBifTADS::getarg,
    &CVmBifTADS::firstobj,
    &CVmBifTADS::nextobj,
    &CVmBifTADS::randomize,
    &CVmBifTADS::rand,
    &CVmBifTADS::cvtstr,
    &CVmBifTADS::cvtnum,
    &CVmBifTADS::gettime,
    &CVmBifTADS::re_match,
    &CVmBifTADS::re_search,
    &CVmBifTADS::re_group,
    &CVmBifTADS::re_replace,
    &CVmBifTADS::savepoint,
    &CVmBifTADS::undo,
    &CVmBifTADS::save,
    &CVmBifTADS::restore,
    &CVmBifTADS::restart,
    &CVmBifTADS::get_max,
    &CVmBifTADS::get_min,
    &CVmBifTADS::make_string
};

/* TADS input/output functions */
void (*G_bif_tadsio[])(VMG_ uint) =
{
    &CVmBifTADS::say,
    &CVmBifTADS::file_open,
    &CVmBifTADS::file_close,
    &CVmBifTADS::file_tell,
    &CVmBifTADS::file_seek,
    &CVmBifTADS::file_seekeof,
    &CVmBifTADS::file_write,
    &CVmBifTADS::file_read,
    &CVmBifTADS::logging,
    &CVmBifTADS::clearscreen,
    &CVmBifTADS::more,
    &CVmBifTADS::input,
    &CVmBifTADS::inputkey,
    &CVmBifTADS::inputevent,
    &CVmBifTADS::inputdialog,
    &CVmBifTADS::askfile,
    &CVmBifTADS::timedelay,
    &CVmBifTADS::sysinfo,
    &CVmBifTADS::status_mode,
    &CVmBifTADS::status_right,
    &CVmBifTADS::res_exists,
    &CVmBifTADS::set_script_file
};

#endif /* VMBIF_DEFINE_VECTOR */
