#ifdef RCSID
static char RCSid[] =
"$Header$";
#endif

/* Copyright (c) 1999 by Michael J. Roberts.  All Rights Reserved. */
/*
Name
  vmmaincl.cpp - T3 VM main entrypoint - command-line version
Function
  This is a command-line entrypoint wrapper for executing a VM program.
  This can be used on command-line OS's (DOS, Unix, etc) where a graphical
  user interface is not needed.

  If a graphical user interface to execute a VM program is required, this
  module should not be used.  Instead, an OS-specific version must be
  written; the OS version obtain the name of the program to run (via a
  file selector or by communication with the graphical shell, for example),
  then must initialize the VM, invoke vm_run_image(), then terminate the
  VM.  The OS version must, in other words, do pretty much everything this
  version does, but should use its own OS-specific means to obtain the name
  of the image file to run rather than reading it from argc/argv.
Notes
  
Modified
  10/07/99 MJRoberts  - Creation
*/

#include <stdlib.h>

#include "t3std.h"
#include "vmmain.h"
#include "vmconsol.h"

/*
 *   Client services interface 
 */
class MyClientIfc: public CVmMainClientIfc
{
public:
    /* pre-execution initialization */
    void pre_exec(struct vm_globals *) { }

    /* post-execution termination/error termination */
    void post_exec(struct vm_globals *) { }
    void post_exec_err(struct vm_globals *) { }

    /* display an error */
    void display_error(struct vm_globals *vmg, const char *msg)
    {
        int old_obey;
        
        /* set up for global access */
        VMGLOB_PTR(vmg);

        /* put the console in obey-whitespace mode for our message */
        old_obey = G_console->set_obey_whitespace(TRUE);

        /* display the message on the console */
        G_console->format_text(vmg_ msg);

        /* restore console mode */
        G_console->set_obey_whitespace(old_obey);
    }
};

/*
 *   Main entrypoint 
 */
int main(int argc, char **argv)
{
    int stat;
    MyClientIfc clientifc;
    int i;

#if 0
{
    static size_t info[] =
    {
        8,8,24,36,8,1039,24,40,16,20,14,14,14,14,14,14,2,8,10,6144,22,1400,
        12,130,28,12,20,20,32,22,456,107,2064,44,320,514,1032,515,
        512,512,512,512,512,512,512,514,1032,515,
        512,512,512,512,512,512,512,98,557,641,28,28,14,47,21,18,18,16,
        15,12,19,12,84,
        20,18,26,260,13,21,19,8,8,23,14,14,14,390,17,104,68,4,28672,4,
        28672,18,65000,45056
    };
    long sum;for (sum = 0, i = 0 ; ; ++i)
    {
        size_t cur = (i >= sizeof(info)/sizeof(info[0]) ? 1024 : info[i]);
        if (malloc(cur) == 0)
            break;
        sum += cur;
        printf("%u ", cur);
    }
    printf("total = %ld, %d of %d\n",
           sum, i, (int)sizeof(info)/sizeof(info[0]));
    exit(0);
}
#endif

    /* 
     *   Check for a "-plain" option; if it's there, set the terminal to
     *   plain text mode.  We must make this check before doing anything
     *   else, because os_plain() must be called prior to os_init() if
     *   it's going to be called at all.  
     */
    for (i = 1 ; i < argc ; ++i)
    {
        if (strcmp(argv[i], "-plain") == 0)
        {
            /* set plain text mode in the OS layer */
            os_plain();

            /* we've found what we're looking for - no need to look further */
            break;
        }
    }
    
    /* 
     *   Initialize the OS layer.  Since this is a command-line-only
     *   implementation, there's no need to ask the OS layer to try to get
     *   us a filename to run, so pass in null for the prompt and filename
     *   buffer.  
     */
    os_init(&argc, argv, 0, 0, 0);

    /* invoke the basic entrypoint */
    stat = vm_run_image_main(&clientifc, "t3run", argc, argv, TRUE, 0);

    /* uninitialize the OS layer */
    os_uninit();

    /* show any unfreed memory */
    t3_list_memory_blocks();

    /* exit with status code */
    os_term(stat);

    /* we shouldn't get here, but in case os_term doesn't really exit... */
    return stat;
}

