/* $Header: d:/cvsroot/tads/tads3/VMPREDEF.H,v 1.2 1999/05/17 02:52:28 MJRoberts Exp $ */

/* Copyright (c) 1998 by Michael J. Roberts.  All Rights Reserved. */
/*
Name
  vmpredef.h - pre-defined objects and properties
Function
  Defines a global structure containing the pre-defined objects
  and properties.  These values are provided by the load image file's
  predefined value symbol table; at load time, we cache these values
  for quick access.
Notes
  
Modified
  12/09/98 MJRoberts  - Creation
*/

#ifndef VMPREDEF_H
#define VMPREDEF_H

#include "t3std.h"
#include "vmtype.h"

/*
 *   pre-defined values 
 */
struct CVmPredef
{
    /* initialize */
    CVmPredef()
    {
        /* the pre-defined things are all undefined initially */
        rterr = VM_INVALID_OBJ;
        rterrmsg_prop = VM_INVALID_PROP;
        obj_construct = VM_INVALID_PROP;
        obj_destruct = VM_INVALID_PROP;
        last_prop = VM_INVALID_PROP;
        obj_call_prop = VM_INVALID_PROP;
        const_str_obj = VM_INVALID_PROP;
        const_lst_obj = VM_INVALID_PROP;
    }

    /* RuntimeError - base exception class for VM errors */
    vm_obj_id_t rterr;

    /* 
     *   getExceptionMessage - property for getting the message from an
     *   exception object 
     */
    vm_prop_id_t rterrmsg_prop;

    /* Constructor - post-construction notifier for TADS objects */
    vm_prop_id_t obj_construct;

    /* Destructor - pre-destruction notifier for TADS objects */
    vm_prop_id_t obj_destruct;

    /* LastProp - last property ID value used by the load image */
    vm_prop_id_t last_prop;

    /* ObjectCallProp - property ID value used for "calling" objects */
    vm_prop_id_t obj_call_prop;

    /* dummy objects representing constant strings and lists */
    vm_obj_id_t const_str_obj;
    vm_obj_id_t const_lst_obj;
};

#endif /* VMPREDEF_H */

